<?php
/**
 *      指定画像のサムネイル画像を自動生成して返します
 */

// 縮小許容サイズ(これ以外の値を受け取った場合は一番近い値に補正)
$resizeList = [50,100,150,200,250,300];

// サムネイルクオリティ(1～100)
$quality = 85;

// サブディレクトリの画像が指定された場合の動作
// (0:カレントディレクトリと同様にサムネイル作成 1:サムネイルは作らず常にオリジナル画像をパススルー表示)
$subDirPathThrough = 1;

// 正方形化で余白埋めする場合の余白の色(カラーコード、カラーネームで指定)
$fillColor = '#ffffff';

/**
 *
 */

// ファイル名パラメータ取得 (通常は .htaccess の設定で自動取得)
if(isset($_GET['f'])) {
    $basename = basename($_GET['f']);
    $dirname = preg_replace('/\.+\/?/', '', dirname($_GET['f']));
    // 本スクリプトが作成したサムネイル保存ディレクトリ名が含まれる場合はnotfoundで終了
    if(preg_match('/thumbnail[\w]+/i', $dirname)) {
        error_exit(404);
    }
    $srcFileName = $dirname == '' ? $basename : "{$dirname}/{$basename}";

    // サブディレクトリパススルー指定の場合、他のパラメータを削除
    if($dirname != '' && $subDirPathThrough) {
        unset($_GET);
    }
} else {
    // ファイル名未指定時はnotfoundで終了
    error_exit(404);
}

// 指定されたファイルが存在しない、指定されたファイル名がpng、jpg、gif以外 いずれかの場合は不正なアクセスとみなしnotfoundで終了
if(!file_exists($srcFileName) || !preg_match('/\.(jpe?g|png|gif)$/i', $srcFileName)) {
    error_exit(404);
}

if(isset($_GET['t'])) {
    // tパラメータ取得
    $s = array_round($resizeList, (int)$_GET['t']);
} else {
    // tパラメータ省略時は0
    $s = 0;
}

// pパラメータ取得
$progressive = '';
if(isset($_GET['p'])) {
    // pパラメータが付加されていた場合、jpeg、pngに限りプログレッシブ/インターレースを有効にする
    $progressive = preg_match('/jpe?g$/i', $srcFileName) ? '-interlace JPEG' :
        (preg_match('/png$/i', $srcFileName) ? '-interlace PNG' : '');
}

// sパラメータ取得
$crop = '';
$fill = '';
$max = 0;
if(isset($_GET['s'])) {
    $crop = 1;
}
// sfパラメータ取得
elseif(isset($_GET['sf'])) {
    $fill = 1;
}

if($s && preg_match('/^\d+$/', $s) || $progressive || $crop || $fill) {
    $thumbnailDirName = sprintf('thumbnail%s', $s ? $s : '_noresize');
    if($dirname) $thumbnailDirName = $dirname. '/'. $thumbnailDirName;
    $newFileName = "./{$thumbnailDirName}/{$basename}";
    if($progressive) $newFileName = preg_replace('/(\.\w+)$/', "_p$1", $newFileName);
    if($crop) $newFileName = preg_replace('/(\.\w+)$/', "_s$1", $newFileName);
    if($fill) $newFileName = preg_replace('/(\.\w+)$/', "_f$1", $newFileName);

    // 対象サイズのサムネイルファイル保存ディレクトリが無い場合はディレクトリ作成
    if(!file_exists("{$thumbnailDirName}")) {
        mkdir("{$thumbnailDirName}", 0755);
    }

    if(file_exists($newFileName)) {
        // 既存のサムネイルよりオリジナルのほうが新しい場合、オリジナルが更新されたとみなし該当サムネイルを削除
        if(filemtime($srcFileName) > filemtime($newFileName) ||
            // 既存のサムネイルよりこのスクリプト自身が新しい場合も、設定変更されたとみなし該当サムネイルを削除
            filemtime(__FILE__) > filemtime($newFileName) ) {
            unlink($newFileName);
        }
    }

    // 縮小済みファイルが無い場合のみ作成
    if(!file_exists($newFileName)) {
        if($crop) {
            list($w, $h) = getimagesize($srcFileName);
            // 画像の中央を正方形に切り出し
            if($w != $h) {
                $min = min([$w, $h]);
                $crop = "-gravity center -crop {$min}x{$min}+0+0";
            }
            else $crop = '';
        }
        if($fill) {
            list($w, $h) = getimagesize($srcFileName);
            // 画像の短辺側を余白色で埋めて正方形化
            if($w != $h) {
                $max = max([$w, $h]);
                $fillColor = preg_replace('/"*?$/', '"', preg_replace('/^"*?/', '"', $fillColor));
                $fill = "-background {$fillColor} -gravity center -extent %dx%d";
                if($s) {
                    $fill = sprintf($fill, $s, $s);
                } else {
                    $fill = sprintf($fill, $max, $max);
                }
            }
            else $fill = '';
        }

        $command = implode(' ', [
            'convert',
            "\"./{$srcFileName}\"",
            $crop,
            $progressive,
            '-auto-orient -strip',
            $quality ? sprintf('-quality %d', $quality) : '',
            $s ? sprintf('-scale %dx%d', $s, $s) : '',
            $fill,
            "\"{$newFileName}\"",
            '']);

        exec($command, $out, $ret);
        // convert実行失敗時はInternalServerErrorで終了
        if($ret != 0) error_exit(500);
        chmod($newFileName, fileperms($srcFileName));
    }
    // 生成したファイル名と更新時刻
    $imgFilename = $newFileName;
    $mtime = filemtime($newFileName);
} else {
    // リサイズしない場合はオリジナルのファイル名と更新時刻
    $imgFilename = $srcFileName;
    $mtime = filemtime($srcFileName);
}

$ifNoneMatch = filter_input(INPUT_SERVER, 'HTTP_IF_NONE_MATCH');
$etag = '"'. md5($mtime. $_SERVER['REQUEST_URI']). '"';
// EtagとリクエストヘッダのIf-None-Matchが異なる場合のみデータを送信、同じ場合はNotModifiedを送ることでブラウザキャッシュを効かせる
if($ifNoneMatch != $etag) {
    header('HTTP/1.1 200 OK');
    header(sprintf('Content-Type: image/%s',
        preg_match('/jpe?g$/i', $srcFileName) ? 'jpeg' : (preg_match('/png$/i', $srcFileName) ? 'png' : 'gif')));
    header("Etag: {$etag}");
    header(sprintf('Last-Modified: %s', gmdate('D, d M Y H:i:s T', $mtime)));
    header('Accept-Ranges: bytes');
    header(sprintf('Content-Length: %d', filesize($imgFilename)));
    readfile($imgFilename);
} else {
    header('HTTP/1.1 304 Not Modified');
    header("Etag: {$etag}");
}

exit;

/**
 *  $arr内から$nに一番近い値を返す
 */
function array_round($arr, $n) {
    $n = (int)$n;
    foreach($arr as $v) $arr_[abs($v - $n)] = $v;
    return $arr_[min(array_keys($arr_))];
}

function error_exit($status = 404) {
    if($status == 404) {
        header('HTTP/1.1 404 Not Found');
        print '<html><body>Not Found</body></html>';
    } else {
        header('HTTP/1.1 500 Internal Server Error');
        print '<html><body>Internal Server Error</body></html>';
    }
    exit;
}
